<?php /**
 * @file
 * Contains \Drupal\oauth_server_sso\Controller\DefaultController.
 */

namespace Drupal\oauth_server_sso\Controller;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Form\FormBuilder;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Response;
use Drupal\oauth_server_sso\DBQueries;
use Drupal\oauth_server_sso\handler;
use Drupal\Component\Utility\Html;
use Drupal\oauth_server_sso\Utilities;

class oauth_server_ssoController extends ControllerBase {

    protected $formBuilder;
    public function __construct(FormBuilder $formBuilder) {
        $this->formBuilder = $formBuilder;
    }

    public static function create(ContainerInterface $container) {
        return new static(
            $container->get("form_builder")
        );
    }

  public static function oauth_server_sso_authorize(){
    $config = \Drupal::config('oauth_server_sso.settings');
    if($config->get('enable_oauth_server') === false){
      Utilities::showErrorMessage('OAuth Server module is not enabled','Please navigate to the <b>Settings</b> tab and enable the <b>Master Switch</b>');
    }
    global $base_url;
    $request = \Drupal::request();
    $client_id = Html::escape($request->get('client_id') ?? '');
    handler::oauth_server_sso_validate_clientId($client_id);
    $redirect_uri = Html::escape($request->get('redirect_uri') ?? '');
    handler::oauth_server_sso_validate_redirectUrl($redirect_uri);
    if(\Drupal::currentUser()->isAuthenticated()){
      $user = \Drupal::currentUser();
      $allUserRolesInTheSystem = user_roles(TRUE);
      $currentUserIsAdmin = FALSE;
      foreach ($user->getRoles() as $roleName) {
       if($allUserRolesInTheSystem[$roleName]->isAdmin()){
         $currentUserIsAdmin = TRUE;
         break;
       }
      }
      if (!$currentUserIsAdmin )
        Utilities::showErrorMessage('Single Sign On not Allowed','This module is exclusively for use by Super Users/Administrators and is for demonstration purposes only.','The Single Sign On feature for the end users is available in the <a target="_blank" href="https://plugins.miniorange.com/drupal-oauth-server"> premium </a> version of the module.');

      $does_exist = DBQueries::get_user_id($user);
      if($does_exist == FALSE){
        DBQueries::insert_user_in_table($user);
      }
      $scope = Html::escape($request->get('scope') ?? '');
      $state = Html::escape($request->get('state') ?? '');
      $nonce = Html::escape($request->get('nonce') ?? '');
      \Drupal::configFactory()->getEditable('oauth_server_sso.settings')->set('nonce',$nonce)->save();
      handler::oauth_server_sso_validate_scope($scope,$user);
      $code = handler::generateRandom(25);
      DBQueries ::insert_code_from_user_id($code, $user);
      if(!empty(\Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_redirect_url'))){
        $url = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_redirect_url');
        if (strpos($url,'?') !== false) {
          $url =$url.'&code='.$code."&state=".$state;
        }else{
          $url =$url.'?code='.$code."&state=".$state;
        }
        \Drupal::logger('oauth_server_sso')->info('<pre><code>'.print_r('auth code sent '.$code, TRUE) .'</code></pre>');
        $code_time = time();
        DBQueries ::insert_code_expiry_from_user_id($code_time,$user);
        $response = new RedirectResponse($url);
        $response->send();
      }else{
        echo "Redirect URL not configured.";
      }
    }else{
     $rem_val = $_SERVER['QUERY_STRING'];
     $redirecting_url = 'mo/oauth2/authorize?'.$rem_val;
     $response = new RedirectResponse($base_url.'/user/login?destination='.urlencode($redirecting_url));
     $response->send();
    }
    return new Response();
  }

  public function oauth_server_sso_access_token(){
    $config = \Drupal::config('oauth_server_sso.settings');
    if($config->get('enable_oauth_server') === false){
      echo json_encode(array(
        "statusCode" => "ERROR",
        "statusMessage" => "OAuth Server module is not enabled",
      )); exit;
    }
    $request = \Drupal::request();
    $request_code = Html::escape($request->get('code') ?? '');
    $token_ep_error=json_encode(array("statusCode" => "ERROR","statusMessage" => "Invalid Grant type or Authorization Code",));
    if(empty($request_code)){
      \Drupal::logger('oauth_server_sso')->warning('Authorization code not received');
      echo $token_ep_error; exit;
    }
    \Drupal::logger('oauth_server_sso')->info('<pre><code>auth code received '.print_r($request_code, TRUE).'</code></pre>');
    $user_details = DBQueries ::get_code_from_user_id($request_code);
    if(!isset($user_details['user_id_val'])){
      \Drupal::logger('oauth_server_sso')->warning('Invalid authorization code received');
      echo $token_ep_error; exit;
    }
    $user =User::load($user_details['user_id_val']);
    if (!$user->hasRole('administrator')) {
       $add_logger=array(
        "statusCode" => 'ERROR',
        "statusMessage" => 'This module is exclusively for use by Super Users/Administrators and is for demonstration purposes only. The Premium version of the module offers the Single Sign-On feature for end users',
       );
       \Drupal::logger('oauth_server_sso')->warning('<pre>code>'.print_r($add_logger, TRUE).'</code></pre>');
        echo json_encode($add_logger); exit;
    }
    handler::validateAuthCode($user_details['auth_code_expiry_time'],$user);
    $redirect_uri = Html::escape($request->get('redirect_uri') ?? '');
    handler::oauth_server_sso_validate_redirectUrl($redirect_uri);
    $client_id = Html::escape($request->get('client_id') ?? $request->server->get('PHP_AUTH_USER') ?? '');
    handler::oauth_server_sso_validate_clientId($client_id);
    $client_secret = Html::escape($request->get('client_secret') ?? $request->server->get('PHP_AUTH_PW') ?? '');
    handler::oauth_server_sso_validate_clientSecret($client_secret);
    $grant_type = Html::escape($request->get('grant_type') ?? '');
    handler::oauth_server_sso_validate_grant($grant_type);
    $access_token = handler::generateRandom(127);
    DBQueries ::insert_access_token_with_user_id($user_details['user_id_val'], $access_token);
    $server_response= array();
    $scope=$user_details['scope'];
    $scope_values = explode(" ", $scope);
    if(in_array('openid', $scope_values) && $config->get('enable_openid') === true){
      $id_token=self::generateIDToken($client_id,$user_details['user_id_val']);
      $server_response = array('id_token' => $id_token,'access_token' => $access_token, 'expires_in' => 3600, 'token_type' => 'Bearer', 'scope' => $scope);
    }else{
      $server_response = array('access_token' => $access_token, 'expires_in' => 3600, 'token_type' => 'Bearer', 'scope' => $scope);
    }
    $req_time = time();
    DBQueries::insert_access_token_expiry_time($user_details['user_id_val'],$req_time);
    \Drupal::logger('oauth_server_sso')->info('<pre><code>token sent '.print_r($server_response, TRUE).'</code></pre>');
    echo json_encode($server_response);exit;
    return new Response();
  }

  public function generateIDToken($client_id,$user_id){
    global $base_url;
    $user_obj = User::load($user_id);
    $uuid=  $user_obj->uuid();
    $email = $user_obj->getEmail();
    $config = \Drupal::config('oauth_server_sso.settings');
    $name = $user_obj->getAccountName();
    $header_arr = [
        'alg' => 'HS256',
        'typ' => 'JWT'
    ];
    $payload_arr = array(
        'iss'        => $base_url.'/mo',
        'sub'        => $uuid,
        'nonce'      => $config->get('nonce'),
        'aud'        => $client_id,
        'iat'        => time(),
        'exp'        => time()+3600,
        'username'   => $name,
        'email'      => $email,
    );
    $base64UrlHeader  = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode( json_encode( $header_arr ) ));
    $base64UrlPayload = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode( json_encode( $payload_arr ) ));
    $data = $base64UrlHeader.'.'.$base64UrlPayload;
    $client_secret = \Drupal::config('oauth_server_sso.settings')->get('oauth_server_sso_client_secret');
    //create signature
    $signature = hash_hmac('sha256', $data, $client_secret);
    $signature = str_replace(['+', '/', '='], ['-', '_', ''], base64_encode($signature));
    $id_token = $data.'.'.$signature;
    return $id_token;
  }

  public function oauth_server_sso_user_info(){
    $config = \Drupal::config('oauth_server_sso.settings');
    if($config->get('enable_oauth_server') === false){
      echo json_encode(array(
        "statusCode" => "ERROR",
        "statusMessage" => "OAuth Server module is not enabled",
      )); exit;
    }
    $request = \Drupal::request();
    $header_value = $request->headers->get('Authorization');
    $access_token_received= (!is_null($header_value))&&(strpos($header_value, 'Bearer ') === 0)?substr($header_value, 7):$request->get('access_token');
    $user_info_error=json_encode(array("statusCode" => "invalid_token","statusMessage" => "Access token is expired or invalid",));
    if(empty($access_token_received)){
      \Drupal::logger('oauth_server_sso')->warning('Access token not received');
      echo $user_info_error; exit;
    }
    \Drupal::logger('oauth_server_sso')->info('<pre><code>access token received '.print_r($access_token_received, TRUE).'</code></pre>');
    $user_details= DBQueries::get_user_id_from_access_token($access_token_received);
    if(!isset($user_details['user_id_val'])){
      \Drupal::logger('oauth_server_sso')->warning('Invalid access token received');
      echo $user_info_error; exit;
    }
    $user =User::load($user_details['user_id_val']);
    if (!$user->hasRole('administrator')) {
      $add_logger=array(
       "statusCode" => 'ERROR',
       "statusMessage" => 'This module is exclusively for use by Super Users/Administrators and is for demonstration purposes only. The Premium version of the module offers the Single Sign-On feature for end users',
      );
      \Drupal::logger('oauth_server_sso')->warning('<pre>code>'.print_r($add_logger, TRUE).'</code></pre>' );
       echo json_encode($add_logger); exit;
    }
    $req_time =$user_details['access_token_request_time'];
    handler::validateAccessToken($req_time);
    $config = \Drupal::config('oauth_server_sso.settings');
    $scope = $user_details['scope'];
    $scope_values = explode(" ", $scope);
    $server_response = array();
    if(in_array('profile', $scope_values)) {
      $server_response= array(
        !empty($config->get('uid_attr_name'))?$config->get('uid_attr_name'):'id'=>$user->id(),
        !empty($config->get('uuid_attr_name'))?$config->get('uuid_attr_name'):'sub'=>$user->uuid(),
        !empty($config->get('name_attr_name'))?$config->get('name_attr_name'):'username'=>$user->getAccountName()
      );
    }
    if(in_array('email', $scope_values)) {
      $server_response = array_merge($server_response,array(
        !empty($config->get('mail_attr_name'))?$config->get('mail_attr_name'):'email'=>$user-> getEmail(),
      ));
    }
    $server_response = array_merge($server_response, ['sub' => $user->uuid()]);
    $genericObject = (object)$server_response;
    \Drupal::logger('oauth_server_sso')->info('<pre><code>userinfo sent '.print_r($genericObject, TRUE).'</code></pre>');
    header('Content-Type: application/json');
    echo json_encode($genericObject);exit;
    return new Response();
  }

  public function downloadPostmanCollection(){
    global $base_url;
    $request = \Drupal::request();

    $configurations = [];
    $configurations['info'] = [
      "_postman_id" => "2821a659-7b56-48be-9e72-ff95a971ee7f",
			"name" => "miniOrange Drupal OAuth Server",
			"schema" => "https://schema.getpostman.com/json/collection/v2.1.0/collection.json"
    ];

    $configurations['item'] = [
        "name" => "Authorization Code Grant",
    ];

    $configurations['item']['request'] = [
      'auth' => [
        'type'   => 'oauth2',
        'oauth2' => Utilities::getOAuthServerConfigurations()
      ],
      "method" => "GET",
      "header" => [],
      "url" => [
        "raw" => $base_url.'/mo/oauth2/userinfo',
        "protocol" => $request->getScheme(),
        "host" => [
          $request->getHost()
        ],
        "path" => [str_replace($request->getSchemeAndHttpHost(), '', $base_url),"mo","oauth2", "userinfo"],
        "query" => []
      ]
    ];

    $configurations['item']['response'] = [];

    header("Content-Disposition: attachment; filename = miniOrange_OAuth_Server_Postman_Collection.json");
    echo(json_encode($configurations, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES));
    exit;
  }

  public function oauthServerDiscoveryEndpointInfo(){
    global $base_url;

    $response_type_supported = ["code"];
    $grant_type_supported = ["Authorization Code Grant"];
    $id_token_signing_alg_values_supported = ["HS256"];
    $scope_supported = ["openid", "email", "profile"];
    $claim_supported = ["iss","sub","aud", "iat","exp","nonce","username", "email"];

    $configurations = [];
    $configurations['issuer'] = $base_url.'/mo';
    $configurations['authorization_endpoint'] = $base_url."/mo/oauth2/authorize";
    $configurations['token_endpoint'] = $base_url."/mo/oauth2/token";
    $configurations['userinfo_endpoint'] = $base_url."/mo/oauth2/token";
    $configurations['jwks_uri'] = $base_url."/mo/oauth2/keys";
    $configurations['response_types_supported'] = $response_type_supported;
    $configurations['grant_types_supported'] = $grant_type_supported;
    $configurations['id_token_signing_alg_values_supported'] = $id_token_signing_alg_values_supported;
    $configurations['scopes_supported'] = $scope_supported;
    $configurations['claims_supported'] = $claim_supported;

    echo "<pre style='white-space: pre-wrap;'>" .json_encode($configurations, JSON_UNESCAPED_SLASHES)."</pre>";
    exit;
  }

  public function openAddNewClientForm(){
    $response = new AjaxResponse();
    $provider_info['add_new_client_info'] = array(
      '#type' => 'item',
      '#markup' => $this->t('<p>You can configure only 1 application in free version of the module. Multiple applications are supported in <a href="licensing">PREMIUM</a> version of module</p>')
    );
    $ajax_form = new OpenModalDialogCommand('Add New OAuth/OpenID Client', $provider_info, ['width' => '40%']);
    $response->addCommand($ajax_form);
    return $response;
  }

  public function openAddMoreCallbackUrlsForm(){
    $response = new AjaxResponse();
    $provider_info['add_new_client_info'] = array(
      '#type' => 'item',
      '#markup' => $this->t('<p>You can add only 1 Callback/Redirect URL in free version of the module. Multiple Callback URLs are supported in <a href="licensing">PREMIUM</a> version of module</p>')
    );
    $ajax_form = new OpenModalDialogCommand('Add Callback URL', $provider_info, ['width' => '40%']);
    $response->addCommand($ajax_form);
    return $response;
  }

  public function openRemoveAccountForm(){
    $response = new AjaxResponse();
    $modal_form = $this->formBuilder->getForm('\Drupal\oauth_server_sso\Form\MiniorangeOAuthRemoveAccount');
    $response->addCommand(new OpenModalDialogCommand('Remove Account', $modal_form, ['width' => '800']));
    return $response;
  }
}
